==> cl_appbase-0.0.23.tar.gz: lib/src/bloc/base_bloc.dart <==
import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:rxdart/rxdart.dart';

import 'collection_controller.dart';
import 'property_state.dart';

/// Base class for business logic components.
///
/// This is inspired by the BLoC pattern but not quite the same.
/// A bloc provides methods that trigger asynchronous procedures
/// which in turn publish state changes through stream controllers.
///
/// A bloc can provide multiple streams to the UI layer.
abstract class BaseBloc {
  final _streamControllers = <StreamController>[];
  final _collectionControllers = <CollectionController>[];
  final _streamSubscriptions = <StreamSubscription>[];

  BaseBloc();

  void initialize();

  /// Instantiates an auto-disposing [BehaviourSubject]
  ///
  /// Disposal is taken care of in the [dispose] method.
  BehaviorSubject<TState> subject<TState>() {
    final subject = BehaviorSubject<TState>();
    _streamControllers.add(subject);
    return subject;
  }

  /// Instantiates an auto-disposing [BehaviourSubject] with
  /// a [PropertyState].
  ///
  /// Disposal is taken care of in the [dispose] method.
  BehaviorSubject<PropertyState<TState>> property<TState>(
      [PropertyState<TState>? initial]) {
    final controller = subject<PropertyState<TState>>();
    if (initial != null) {
      controller.sink.add(initial);
    }
    return controller;
  }

  /// Instantiates an auto-disposing [CollectionController].
  ///
  /// Disposal is taken care of in the [dispose] method.
  CollectionController<TItem> collection<TItem>(
      Future<Iterable<TItem>> Function(int offset, int limit) query,
      {int chunkSize = 25}) {
    final controller = CollectionController<TItem>(query, chunkSize: chunkSize);
    _collectionControllers.add(controller);
    return controller;
  }

  /// Instantiates an auto-disposing [StreamController].
  ///
  /// Disposal is taken care of in the [dispose] method.
  StreamController<TState> stream<TState>() {
    final subject = StreamController<TState>();
    _streamControllers.add(subject);
    return subject;
  }

  /// Register a [StreamSubscription] for auto-disposal.
  ///
  /// Disposal is taken care of in the [dispose] method.
  void sub(StreamSubscription subscription) {
    _streamSubscriptions.add(subscription);
  }

  @mustCallSuper
  void dispose() {
    // close all state stream controllers
    _streamControllers.forEach((s) {
      s.close();
    });
    // dispose all collection controllers
    _collectionControllers.forEach((s) {
      s.dispose();
    });
    // cancel all prerecorded stream subscriptions
    _streamSubscriptions.forEach((s) {
      s.cancel();
    });
  }
}

==> cl_appbase-0.0.28.tar.gz: lib/src/bloc/base_bloc.dart <==
import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:rxdart/rxdart.dart';

import 'collection_controller.dart';
import 'property_state.dart';

/// Base class for business logic components.
///
/// This is inspired by the BLoC pattern but not quite the same.
/// A bloc provides methods that trigger asynchronous procedures
/// which in turn publish state changes through stream controllers.
///
/// A bloc can provide multiple streams to the UI layer.
abstract class BaseBloc {
  final _streamControllers = <StreamController>[];
  final _collectionControllers = <CollectionController>[];
  final _streamSubscriptions = <StreamSubscription>[];

  BaseBloc();

  void initialize() {}

  /// Instantiates an auto-disposing [BehaviourSubject]
  ///
  /// Disposal is taken care of in the [dispose] method.
  BehaviorSubject<TState> subject<TState>([TState? initial]) {
    final subject = BehaviorSubject<TState>();
    if (initial != null) {
      subject.sink.add(initial);
    }
    _streamControllers.add(subject);
    return subject;
  }

  /// Instantiates an auto-disposing [BehaviourSubject] with
  /// a [PropertyState].
  ///
  /// Disposal is taken care of in the [dispose] method.
  BehaviorSubject<PropertyState<TState>> property<TState>(
      PropertyState<TState> initial) {
    final controller = subject<PropertyState<TState>>();
    controller.sink.add(initial);
    return controller;
  }

  /// Instantiates an auto-disposing [CollectionController].
  ///
  /// Disposal is taken care of in the [dispose] method.
  CollectionController<TItem> collection<TItem>(
      Future<Iterable<TItem>> Function(int offset, int limit) query,
      {int chunkSize = 25}) {
    final controller = CollectionController<TItem>(query, chunkSize: chunkSize);
    _collectionControllers.add(controller);
    return controller;
  }

  /// Instantiates an auto-disposing [StreamController].
  ///
  /// Disposal is taken care of in the [dispose] method.
  StreamController<TState> stream<TState>() {
    final subject = StreamController<TState>();
    _streamControllers.add(subject);
    return subject;
  }

  /// Register a [StreamSubscription] for auto-disposal.
  ///
  /// Disposal is taken care of in the [dispose] method.
  void sub(StreamSubscription subscription) {
    _streamSubscriptions.add(subscription);
  }

  @mustCallSuper
  void dispose() {
    // close all state stream controllers
    _streamControllers.forEach((s) {
      s.close();
    });
    // dispose all collection controllers
    _collectionControllers.forEach((s) {
      s.dispose();
    });
    // cancel all prerecorded stream subscriptions
    _streamSubscriptions.forEach((s) {
      s.cancel();
    });
  }
}
